@@ -5,6 +5,8 @@ import logging
 import re
 import xml.etree.ElementTree as ET
 import zipfile
+import mimetypes
+import urllib
 
 from django.core.files.base import ContentFile
 from django.core.files.storage import default_storage
@@ -13,15 +15,22 @@ from django.template import Context, Template
 from django.utils import timezone
 from django.utils.module_loading import import_string
 from webob import Response
-import pkg_resources
 from six import string_types
-
 from web_fragments.fragment import Fragment
 from xblock.core import XBlock
 from xblock.completable import CompletableXBlockMixin
 from xblock.exceptions import JsonHandlerError
 from xblock.fields import Scope, String, Float, Boolean, Dict, DateTime, Integer
 
+try:
+    # Older Open edX releases (Redwood and earlier) install a backported version of
+    # importlib.resources: https://pypi.org/project/importlib-resources/
+    import importlib_resources
+except ModuleNotFoundError:
+    # Starting with Sumac, Open edX drops importlib-resources in favor of the standard library:
+    # https://docs.python.org/3/library/importlib.resources.html#module-importlib.resources
+    from importlib import resources as importlib_resources
+
 try:
     try:
         from common.djangoapps.student.models import CourseEnrollment
@@ -40,6 +49,7 @@ def _(text):
 
 
 logger = logging.getLogger(__name__)
+OS_PATH_ALT_SEP = '\\'
 
 
 @XBlock.wants("settings")
@@ -65,7 +75,7 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
     By default, static assets are stored in the default Django storage backend. To
     override this behaviour, you should define a custom storage function. This
     function must take the xblock instance as its first and only argument. For instance,
-    you can store assets in different directories depending on the XBlock organisation with::
+    you can store assets in different directories depending on the XBlock organization with::
 
         def scorm_storage(xblock):
             from django.conf import settings
@@ -146,6 +156,24 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
         default=False,
         scope=Scope.settings,
     )
+    enable_navigation_menu = Boolean(
+        display_name=_("Display navigation menu"),
+        help=_(
+            "Select True to display a navigation menu on the left side to display table of contents"
+        ),
+        default=False,
+        scope=Scope.settings,
+    )
+
+    navigation_menu = String(scope=Scope.settings, default="")
+
+    navigation_menu_width = Integer(
+        display_name=_("Display width of navigation menu(px)"),
+        help=_(
+            "Width of navigation menu. This assumes that Navigation Menu is enabled. (default: 30%)"
+        ),
+        scope=Scope.settings,
+    )
 
     has_author_view = True
 
@@ -160,30 +188,40 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
     def get_current_user(self):
         return self.runtime.service(self, "user").get_current_user()
 
+    def initialize_student_info(self):
+        user_id = self.get_current_user_attr("edx-platform.user_id")
+        username = self.get_current_user_attr("edx-platform.username")
+        
+        self.scorm_data["cmi.core.student_id"] = user_id
+        self.scorm_data["cmi.learner_id"] = user_id
+        self.scorm_data["cmi.learner_name"] = username
+        self.scorm_data["cmi.core.student_name"] = username
+
     @staticmethod
     def resource_string(path):
         """Handy helper for getting static resources from our kit."""
-        data = pkg_resources.resource_string(__name__, path)
+        data = importlib_resources.files(__name__).joinpath(path).read_bytes()
         return data.decode("utf8")
 
     def author_view(self, context=None):
         context = context or {}
         if not self.index_page_path:
-            context[
-                "message"
-            ] = "Click 'Edit' to modify this module and upload a new SCORM package."
+            context["message"] = "Click 'Edit' to modify this module and upload a new SCORM package."
         context["can_view_student_reports"] = True
         return self.student_view(context=context)
 
     def student_view(self, context=None):
         student_context = {
-            "index_page_url": self.index_page_url,
+            "index_page_url": urllib.parse.unquote(self.index_page_url),
             "completion_status": self.lesson_status,
             "grade": self.get_grade(),
             "can_view_student_reports": self.can_view_student_reports,
             "scorm_xblock": self,
+            "navigation_menu": self.navigation_menu,
+            "popup_on_launch": self.popup_on_launch,
         }
         student_context.update(context or {})
+        self.initialize_student_info()
         template = self.render_template("static/html/scormxblock.html", student_context)
         frag = Fragment(template)
         frag.add_css(self.resource_string("static/css/scormxblock.css"))
@@ -202,6 +240,33 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
         )
         return frag
 
+    @XBlock.handler
+    def assets_proxy(self, request, suffix):
+        """
+        Proxy view for serving assets. It receives a request with the path to the asset to serve, generates a pre-signed
+        URL to access the content in the AWS S3 bucket, and returns a redirect response to the pre-signed URL.
+
+        Parameters:
+        ----------
+        request : django.http.request.HttpRequest
+            HTTP request object containing the path to the asset to serve.
+        suffix : str
+            The part of the URL after 'assets_proxy/', i.e., the path to the asset to serve.
+
+        Returns:
+        -------
+        Response object containing the content of the requested file with the appropriate content type.
+        """
+        file_name = os.path.basename(suffix)
+        signed_url = self.storage.url(suffix)
+        if request.query_string:
+            signed_url = "&".join([signed_url, request.query_string])
+        file_type, _ = mimetypes.guess_type(file_name)
+        with urllib.request.urlopen(signed_url) as response:
+            file_content = response.read()
+
+        return Response(file_content, content_type=file_type)
+
     def studio_view(self, context=None):
         # Note that we cannot use xblockutils's StudioEditableXBlockMixin because we
         # need to support package file uploads.
@@ -212,6 +277,9 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
             "field_width": self.fields["width"],
             "field_height": self.fields["height"],
             "field_popup_on_launch": self.fields["popup_on_launch"],
+            "field_enable_navigation_menu": self.fields["enable_navigation_menu"],
+            "field_navigation_menu_width": self.fields["navigation_menu_width"],
+            "popup_on_launch": self.fields["popup_on_launch"],
             "scorm_xblock": self,
         }
         studio_context.update(context or {})
@@ -234,6 +302,10 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
         self.width = parse_int(request.params["width"], None)
         self.height = parse_int(request.params["height"], None)
         self.has_score = request.params["has_score"] == "1"
+        self.enable_navigation_menu = request.params["enable_navigation_menu"] == "1"
+        self.navigation_menu_width = parse_int(
+            request.params["navigation_menu_width"], None
+        )
         self.weight = parse_float(request.params["weight"], 1)
         self.popup_on_launch = request.params["popup_on_launch"] == "1"
         self.icon_class = "problem" if self.has_score else "video"
@@ -269,12 +341,15 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
                 "index_page_url": self.index_page_url,
                 "width": self.width or 800,
                 "height": self.height or 800,
+                "navigation_menu": self.navigation_menu,
+                "navigation_menu_width": self.navigation_menu_width,
+                "enable_navigation_menu": self.enable_navigation_menu,
             },
         )
         return Response(body=rendered)
 
     def clean_storage(self):
-        if self.storage.exists(self.extract_folder_base_path):
+        if self.path_exists(self.extract_folder_base_path):
             logger.info(
                 'Removing previously unzipped "%s"', self.extract_folder_base_path
             )
@@ -317,7 +392,9 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
                     # the is_dir() method to verify whether a ZipInfo object points to a
                     # directory.
                     # https://docs.python.org/3.6/library/zipfile.html#zipfile.ZipInfo.is_dir
-                    if not zipinfo.filename.endswith("/"):
+                    # TODO: remove backported 'is_dir' method once upgraded to 
+                    # python 3.12.3 or greater. 
+                    if not is_dir(zipinfo):
                         dest_path = os.path.join(
                             self.extract_folder_path,
                             os.path.relpath(zipinfo.filename, root_path),
@@ -333,12 +410,13 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
             return ""
         folder = self.extract_folder_path
         if self.storage.exists(
-            os.path.join(self.extract_folder_base_path, self.index_page_path)
+            os.path.join(self.extract_folder_base_path, self.clean_path(self.index_page_path))
         ):
             # For backward-compatibility, we must handle the case when the xblock data
             # is stored in the base folder.
             folder = self.extract_folder_base_path
             logger.warning("Serving SCORM content from old-style path: %s", folder)
+
         return self.storage.url(os.path.join(folder, self.index_page_path))
 
     @property
@@ -349,25 +427,67 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
         """
         return os.path.join(self.extract_folder_base_path, self.package_meta["sha1"])
 
+    def clean_path(self, path):
+        """
+        Removes query string from a path
+        """
+        return path.split('?')[0] if path else path
+    
+    def path_exists(self, path):
+        """
+        Returs True if given path exists in storage otherwise returns False
+        """
+        try:
+            dirs, files = self.storage.listdir(path)
+            return True if dirs or files else False
+        except FileNotFoundError:
+            return False
+
     @property
     def extract_folder_base_path(self):
         """
         Path to the folder where packages will be extracted.
+        Compute hash of the unique block usage_id and use that as our directory name.
+        """
+        # For backwards compatibility, we return the old path if the directory exists
+        if self.path_exists(self.extract_old_folder_base_path):
+            return self.extract_old_folder_base_path
+        sha1 = hashlib.sha1()
+        sha1.update(str(self.scope_ids.usage_id).encode())
+        hashed_usage_id = sha1.hexdigest()
+        return os.path.join(self.scorm_location(), hashed_usage_id)
+    
+    @property
+    def extract_old_folder_base_path(self):
+        """
+        Deprecated Path to the folder where packages will be extracted.
+        Deprecated as the block_id was shared by courses when exporting and importing
+        them, thus causing storage conflicts.
+        Only keeping this here for backwards compatibility.
         """
         return os.path.join(self.scorm_location(), self.location.block_id)
 
+    def get_mode(self, data):
+        if "preview" in data["url"]:
+            return "review"
+        return "normal"
+
     @XBlock.json_handler
     def scorm_get_value(self, data, _suffix):
         """
         Here we get only the get_value events that were not filtered by the LMSGetValue js function.
         """
         name = data.get("name")
+        if name in ["cmi.core.lesson_mode", "cmi.mode"]:
+            return {"value": self.get_mode(data)}
         if name in ["cmi.core.lesson_status", "cmi.completion_status"]:
             return {"value": self.lesson_status}
         if name == "cmi.success_status":
             return {"value": self.success_status}
         if name in ["cmi.core.score.raw", "cmi.score.raw"]:
             return {"value": self.lesson_score * 100}
+        if name == "cmi.score.scaled":
+            return {"value": self.lesson_score}
         if name in ["cmi.core.student_id", "cmi.learner_id"]:
             return {"value": self.get_current_user_attr("edx-platform.user_id")}
         if name in ["cmi.core.student_name", "cmi.learner_name"]:
@@ -393,9 +513,11 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
         completion_status = None
         lesson_score = None
 
+        is_completed = self.lesson_status == "completed"
+
         self.scorm_data[name] = value
         if name == "cmi.core.lesson_status":
-            lesson_status = data.get("value")
+            lesson_status = value
             if lesson_status in ["passed", "failed"]:
                 success_status = lesson_status
             elif lesson_status in ["completed", "incomplete"]:
@@ -406,6 +528,8 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
             completion_status = value
         elif name in ["cmi.core.score.raw", "cmi.score.raw"] and self.has_score:
             lesson_score = parse_validate_positive_float(value, name) / 100.0
+        elif name == "cmi.score.scaled" and self.has_score:
+            lesson_score = parse_validate_positive_float(value, name)
         elif name == "cmi.progress_measure":
             completion_percent = parse_validate_positive_float(value, name)
 
@@ -422,7 +546,11 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
             self.success_status = success_status
         if completion_status == "completed":
             self.emit_completion(1)
-        if success_status or completion_status == "completed":
+        if (
+            success_status
+            or completion_status == "completed"
+            or (is_completed and lesson_score)
+        ):
             if self.has_score:
                 self.publish_grade()
 
@@ -484,12 +612,14 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
 
         prefix = "{" + namespace + "}" if namespace else ""
         resource = root.find(
-            "{prefix}resources/{prefix}resource[@href]".format(prefix=prefix)
+            f"{prefix}resources/{prefix}resource[@href]"
         )
         schemaversion = root.find(
-            "{prefix}metadata/{prefix}schemaversion".format(prefix=prefix)
+            f"{prefix}metadata/{prefix}schemaversion"
         )
 
+        self.extract_navigation_titles(root, prefix)
+
         if resource is not None:
             self.index_page_path = resource.get("href")
         else:
@@ -501,6 +631,123 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
         else:
             self.scorm_version = "SCORM_12"
 
+    def extract_navigation_titles(self, root, prefix):
+        """Extracts all the titles of items to build a navigation menu from the imsmanifest.xml file
+
+        Args:
+            root (XMLTag): root of the imsmanifest.xml file
+            prefix (string): namespace to match with in the xml file
+        """
+        organizations = root.findall(
+            f"{prefix}organizations/{prefix}organization"
+        )
+        navigation_menu_titles = []
+        # Get data for all organizations
+        for organization in organizations:
+            navigation_menu_titles.append(
+                self.find_titles_recursively(organization, prefix, root)
+            )
+        self.navigation_menu = self.recursive_unorderedlist(navigation_menu_titles)
+
+    def sanitize_input(self, input_str):
+        """Removes script tags from string"""
+        sanitized_str = re.sub(
+            r"<script\b[^>]*>(.*?)</script>", "", input_str, flags=re.IGNORECASE
+        )
+        return sanitized_str
+
+    def find_titles_recursively(self, item, prefix, root):
+        """Recursively iterate through the organization tags and extract the title and resources
+
+        Args:
+            item (XMLTag): The current node to iterate on
+            prefix (string): namespace to match with in the xml file
+            root (XMLTag): root of the imsmanifest.xml file
+
+        Returns:
+            List: Nested list of all the title tags and their resources
+        """
+        children = item.findall(f"{prefix}item")
+        item_title = item.find(f"{prefix}title").text
+        # Sanitizing every title tag to protect against XSS attacks
+        sanitized_title = self.sanitize_input(item_title)
+        item_identifier = item.get("identifierref")
+        # If item does not have a resource, we don't need to make it into a link
+        if not item_identifier:
+            resource_link = "#"
+        else:
+            resource = root.find(
+                f"{prefix}resources/{prefix}resource[@identifier='{item_identifier}']"
+            )
+            # Attach the storage path with the file path
+            resource_link = urllib.parse.unquote(
+                self.storage.url(
+                    os.path.join(self.extract_folder_path, resource.get("href"))
+                )
+            )
+        if not children:
+            return [(sanitized_title, resource_link)]
+        child_titles = []
+        for child in children:
+            if "isvisible" in child.attrib and child.attrib["isvisible"] == "true":
+                child_titles.extend(self.find_titles_recursively(child, prefix, root))
+        return [(sanitized_title, resource_link), child_titles]
+
+    def recursive_unorderedlist(self, value):
+        """Create an HTML unordered list recursively to display navigation menu
+
+        Args:
+            value (list): The nested list to create the unordered list
+        """
+
+        def has_children(item):
+            return len(item) == 2 and (type(item[0]) is tuple and type(item[1]) is list)
+
+        def format(items, tabs=1):
+            """Iterate through the nested list and return a formatted unordered list"""
+            indent = "\t" * tabs
+            # If leaf node, return the li tag
+            if type(items) is tuple:
+                title, resource_url = items[0], items[1]
+                if resource_url != "#":
+                    return f"{indent}<li href='{resource_url}' class='navigation-title'>{title}</li>"
+
+                return f"{indent}<li class='navigation-title-header'>{title}</li>"
+
+            output = []
+            # If parent node, create another nested unordered list and return
+            if has_children(items):
+                parent, children = items[0], items[1]
+                title, resource_url = parent[0], parent[1]
+                for child in children:
+                    output.append(format(child, tabs + 1))
+                if resource_url != "#":
+                    return "\n{indent}<ul>\n{indent}<li href='{resource_url}' class='navigation-title'>{title}</li>\n{indent}<ul>\n{indent}\n{output}</ul>\n{indent}</ul>".format(
+                        indent=indent,
+                        resource_url=resource_url,
+                        title=title,
+                        output="\n".join(output),
+                    )
+                return "\n{indent}<ul>\n{indent}<li class='navigation-title-header'>{title}</li>\n{indent}<ul>\n{indent}\n{output}</ul>\n{indent}</ul>".format(
+                    indent=indent,
+                    resource_url=resource_url,
+                    title=title,
+                    output="\n".join(output),
+                )
+            else:
+                for item in items:
+                    output.append(format(item, tabs + 1))
+                return "{indent}\n{indent}<ul>\n{output}\n{indent}</ul>".format(
+                    indent=indent, output="\n".join(output)
+                )
+
+        unordered_lists = []
+        # Append navigation menus for all organizations in course
+        for organization in value:
+            unordered_lists.append(format(organization))
+
+        return "\n".join(unordered_lists)
+
     def find_relative_file_path(self, filename):
         return os.path.relpath(self.find_file_path(filename), self.extract_folder_path)
 
@@ -511,9 +758,7 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
         """
         path = self.get_file_path(filename, self.extract_folder_path)
         if path is None:
-            raise ScormError(
-                "Invalid package: could not find '{}' file".format(filename)
-            )
+            raise ScormError(f"Invalid package: could not find '{filename}' file")
         return path
 
     def get_file_path(self, filename, root):
@@ -594,9 +839,7 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
             [
                 {
                     "data": {"student_id": enrollment.user.id},
-                    "value": "{} ({})".format(
-                        enrollment.user.username, enrollment.user.email
-                    ),
+                    "value": f"{enrollment.user.username} ({enrollment.user.email})"
                 }
                 for enrollment in enrollments[:20]
             ]
@@ -611,7 +854,7 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
             user_id = int(user_id)
         except (TypeError, ValueError):
             return Response(
-                body="Invalid 'id' parameter {}".format(user_id), status=400
+                body=f"Invalid 'id' parameter {user_id}", status=400
             )
         try:
             module = StudentModule.objects.filter(
@@ -621,7 +864,7 @@ class ScormXBlock(XBlock, CompletableXBlockMixin):
             ).get()
         except StudentModule.DoesNotExist:
             return Response(
-                body="No data found for student id={}".format(user_id),
+                body=f"No data found for student id={user_id}",
                 status=404,
             )
         except StudentModule.MultipleObjectsReturned:
@@ -703,12 +946,25 @@ def parse_validate_positive_float(value, name):
         parsed = float(value)
     except (TypeError, ValueError):
         raise ValueError(
-            "Could not parse value of '{}' (must be float): {}".format(name, value)
+            f"Could not parse value of '{name}' (must be float): {value}"
         )
     if parsed < 0:
-        raise ValueError("Value of '{}' must not be negative: {}".format(name, value))
+        raise ValueError(f"Value of '{name}' must not be negative: {value}")
     return parsed
 
 
+def is_dir(zipinfo):
+    """Return True if this archive member is a directory."""
+    if zipinfo.filename.endswith('/'):
+        return True
+    elif zipinfo.filename.endswith((os.path.sep, OS_PATH_ALT_SEP)):
+        # The ZIP format specification requires to use forward slashes
+        # as the directory separator, but in practice some ZIP files
+        # created on Windows can use backward slashes.  For compatibility
+        # with the extraction code which already handles this:
+        return True
+    return False 
+
+
 class ScormError(Exception):
     pass
