@@ -93,6 +93,9 @@ function ScormXBlock(runtime, element, settings) {
     // Student reports
     var reportElement = $(element).find(".scorm-reports .report");
     function initReports() {
+        if (reportElement.length === 0){
+            return;
+        }
         $(element).find("button.view-reports").on("click", function () {
             viewReports();
         });
@@ -159,6 +162,14 @@ function ScormXBlock(runtime, element, settings) {
         });
     }
 
+    // Flag to verify if navigation menu was used to change page
+    var navigationClick = false;
+        $(element).on("click",".navigation-title", function () {
+            var path = $(this).attr('href');
+            $(element).find('.scorm-embedded').attr('src', path);
+            navigationClick = true;
+         });
+        
     // We only make calls to the get_value handler when absolutely required.
     // These calls are synchronous and they can easily clog the scorm display.
     var uncachedValues = [
@@ -166,27 +177,37 @@ function ScormXBlock(runtime, element, settings) {
         "cmi.completion_status",
         "cmi.success_status",
         "cmi.core.score.raw",
-        "cmi.score.raw"
+        "cmi.score.raw",
+        "cmi.score.scaled",
+        "cmi.mode"
     ];
     var getValueUrl = runtime.handlerUrl(element, 'scorm_get_value');
-    var GetValue = function (cmi_element) {
-        if (cmi_element in uncachedValues) {
-            var response = $.ajax({
+    var GetValue = function (cmi_element) {   
+        // Only make a call if navigation menu was not used
+        // Otherwise the synchronous calls are blocked by chromium on page unload
+        if (uncachedValues.includes(cmi_element) && !navigationClick){
+            $.ajax({
                 type: "POST",
                 url: getValueUrl,
                 data: JSON.stringify({
-                    'name': cmi_element
+                    'name': cmi_element,
+                    'url': window.location.href
                 }),
-                async: false
+                async: false,
+                success: function (response) {
+                    // Set to false to allow for other calls by the SCORM api
+                    navigationClick = false;
+                    return response.value;
+                }
             });
-            response = JSON.parse(response.responseText);
-            return response.value;
         } else if (cmi_element in settings.scorm_data) {
+            navigationClick = false;
             return settings.scorm_data[cmi_element];
         }
+        navigationClick = false;
         return "";
     };
-
+    
     var setValueEvents = [];
     var processingSetValueEventsQueue = false;
     var setValuesUrl = runtime.handlerUrl(element, 'scorm_set_values');
@@ -213,7 +234,7 @@ function ScormXBlock(runtime, element, settings) {
             params = setValueEvents.shift();
             cmi_element = params[0];
             value = params[1];
-            if (!cmi_element in uncachedValues) {
+            if (!uncachedValues.includes(cmi_element)) {
                 // Update the local scorm data copy to fetch results faster with get_value
                 settings.scorm_data[cmi_element] = value;
             }
