@@ -3,7 +3,7 @@ import json
 import unittest
 
 
-from ddt import ddt, data, unpack
+from ddt import ddt, data
 from freezegun import freeze_time
 import mock
 from xblock.field_data import DictFieldData
@@ -129,9 +129,7 @@ class ScormXBlockTests(unittest.TestCase):
     )
     @mock.patch("openedxscorm.scormxblock.default_storage")
     def test_student_view_data(self, default_storage, file_storage_path):
-        block = self.make_one(
-            package_meta={"last_updated": "2018-05-01", "size": 1234}
-        )
+        block = self.make_one(package_meta={"last_updated": "2018-05-01", "size": 1234})
         default_storage.configure_mock(url=mock.Mock(return_value="url_zip_file"))
 
         student_view_data = block.student_view_data()
@@ -245,7 +243,8 @@ class ScormXBlockTests(unittest.TestCase):
         self.assertEqual(response.json, {"value": "status"})
 
     @data(
-        {"name": "cmi.core.score.raw"}, {"name": "cmi.score.raw"},
+        {"name": "cmi.core.score.raw"},
+        {"name": "cmi.score.raw"},
     )
     def test_scorm_get_lesson_score(self, value):
         block = self.make_one(lesson_score=0.2)
@@ -256,68 +255,14 @@ class ScormXBlockTests(unittest.TestCase):
 
         self.assertEqual(response.json, {"value": 20})
 
-    @data(
-        {"name": "cmi.core.lesson_location"},
-        {"name": "cmi.location"},
-        {"name": "cmi.suspend_data"},
-    )
-    def test_get_other_scorm_values(self, value):
-        block = self.make_one(
-            scorm_data={
-                "cmi.core.lesson_location": 1,
-                "cmi.location": 2,
-                "cmi.suspend_data": [1, 2],
-            }
-        )
+    def test_scorm_data_has_user_info_in_student_view(self):
+        block = self.make_one()
 
-        response = block.scorm_get_value(
-            mock.Mock(method="POST", body=json.dumps(value))
-        )
-
-        self.assertEqual(response.json, {"value": block.scorm_data[value["name"]]})
-
-    @data(
-        ({'name': 'cmi.core.student_id'}, 'edx-platform.user_id', 23),
-        ({'name': 'cmi.core.student_name'}, 'edx-platform.username', 'supername')
-    )
-    @unpack
-    def test_scorm_12_get_student_data(self, request_data, key, value):
-        service_user_mock = mock.Mock()
-        current_user_mock = mock.Mock()
-        current_user_mock.opt_attrs = {
-            key : value
-        }
-        service_user_mock.configure_mock(**{'get_current_user.return_value': current_user_mock})
-
-        runtime = mock.Mock()
-        runtime.service.return_value = service_user_mock
-
-        block = self.make_one(runtime=runtime)
-
-        response = block.scorm_get_value(
-            mock.Mock(method="POST", body=json.dumps(request_data))
-        )
-        self.assertEqual(response.json, {'value': value})
-        
-    @data(
-        ({'name': 'cmi.learner_id'}, 'edx-platform.user_id', 23),
-        ({'name': 'cmi.learner_name'}, 'edx-platform.username', 'supername')
-    )
-    @unpack
-    def test_scorm_2004_get_student_data(self, request_data, key, value):
-        service_user_mock = mock.Mock()
-        current_user_mock = mock.Mock()
-        current_user_mock.opt_attrs = {
-            key : value
-        }
-        service_user_mock.configure_mock(**{'get_current_user.return_value': current_user_mock})
-
-        runtime = mock.Mock()
-        runtime.service.return_value = service_user_mock
-
-        block = self.make_one(runtime=runtime)
-
-        response = block.scorm_get_value(
-            mock.Mock(method="POST", body=json.dumps(request_data))
-        )
-        self.assertEqual(response.json, {'value': value})    
\ No newline at end of file
+        block.student_view()
+        student_info_keys = [
+            "cmi.core.student_id",
+            "cmi.learner_id",
+            "cmi.learner_name",
+            "cmi.core.student_name",
+        ]
+        self.assertTrue(key in block.scorm_data for key in student_info_keys)
